#!/bin/bash 

# install.sh
# Install EMG patches log results for LMI and SiteProtector
#

#set -x

SUCCESS="0"
NOT_FOUND="NOT_FOUND";
PATCH_CONF="/etc/crm/patches.config"
DEFAULT_PATCH_ARCHIVE_DIR=/etc/lmi/patches

# Read patch.conf to get value corresponding to input name
# - for example:  GetConfigValue "PatchArchiveDir=" PATCH_DIR ==> sets PATCH_DIR to "/etc/lmi/patches" 
# - returns NOT_FOUND if no corresponding key is found
GetConfigValue()
{
	KEY="$1"
	if [ -f $PATCH_CONF ] ; then
		VALUE=`cat $PATCH_CONF | sed -n "/$KEY/p" | sed "s/$KEY//"`
	fi 
	if [ x"" == x"$VALUE" ] ; then
		VALUE=$NOT_FOUND
	fi
	eval "$2=$VALUE"
}

# Determine patch ID (*.info) file in patch contents in current dir
GetInfoFilename()
{
	NUM_FILE=`find . -maxdepth 1 -regex '^\./patch.*\.info' -print | wc -l`
	if [ x"1" != x"$NUM_FILE" ] ; then
		echo "More than one *.info file exists in current directory---cannot determine patch ID."
		echo "Please extract patch to empty directory and try again."
		exit 1;
	else
		INFO=`find . -maxdepth 1 -regex '^\./patch.*\.info' -print`
	fi
	eval "$1=$INFO"
}

# Determine patch ID (*.info) file in patch contents in current dir
GetPatchID()
{
	GetInfoFilename INFO_FILE 
	# INFO_FILE should be something like ./patch285.info
	ID=`echo "$INFO_FILE" | sed s/[^0-9]//g`
	if [ x"" == x"$ID" ] ; then
		echo "Unable to parse patch number from *.info file:  $INFO_FILE.  Exiting."
		exit 1;
	fi
	eval "$1=$ID"
}

GetPatchID PATCH_ID
STATUS_WARN=0
STATUS_ERR=0

MODE="install"

Log() { echo $* ; }

LogError()
{
    Log ERROR: $*
    STATUS_ERR=1
}

LogWarning()
{
    Log WARNING: $*
    STATUS_WARN=1
}

# ProvG Helper Functions
KillDaemon()
{
  if ps -e | grep issDaemon >/dev/null 2>&1; then
    Log "Stopping ISS Daemon."
    /etc/init.d/issDaemon stop >/dev/null 2>&1
  fi
}

Install()
{

    KillDaemon

    INSTALL_LIST=
    for i in $@ ; do
        PKGVER=`rpm -qp $i`
        BASE=`basename $i -new.rpm`
        if rpm -q $BASE >/dev/null 2>&1 ; then
            INSTVER=`rpm -q $BASE`
            if iss-rpm-compare $i ; then
                Log Upgrading $INSTVER to $PKGVER
                if [ -f $BASE-new.sigtgz ] ; then 
                    tar -C / -xzf $BASE-new.sigtgz || LogError Failed to extract signatures for $BASE
                fi
                INSTALL_LIST="$i $INSTALL_LIST"
            else
                LogWarning Skipping $PKGVER, $INSTVER already installed
            fi
        else
            Log Installing $PKGVER
            if [ -f $BASE-new.sigtgz ] ; then
                tar -C / -xzf $BASE-new.sigtgz || LogError Failed to extract signatures for $BASE
            fi
            INSTALL_LIST="$i $INSTALL_LIST"
        fi
    done
    if [ ! -z "$INSTALL_LIST" ] ; then
        rpm $RPM_INSTALL_OPTS $INSTALL_LIST && return 0 || LogError Failed while installing packages $INSTALL_LIST
    fi
    return 1
}

Uninstall()
{

    KillDaemon

    INSTALL_LIST=
    for i in $@ ; do
        PKGVER=`rpm -qp $i`
        BASE=`basename $i -original.rpm`
        if rpm -q $BASE >/dev/null 2>&1 ; then
            INSTVER=`rpm -q $BASE`
            #if iss-rpm-compare $i ; then
                Log Downgrading $INSTVER to $PKGVER
                if [ -f $BASE-original.sigtgz ] ; then
                    tar -C / -xzf $BASE-original.sigtgz || LogError Failed to extract signatures for $BASE
                fi
                INSTALL_LIST="$i $INSTALL_LIST"
            #else
                #LogWarning Skipping $PKGVER, $INSTVER already installed
            #fi
        else
            Log Installing $PKGVER
            if [ -f $BASE-original.sigtgz ] ; then
                tar -C / -xzf $BASE-original.sigtgz || LogError Failed to extract signatures for $BASE
            fi
            INSTALL_LIST="$i $INSTALL_LIST"
        fi
    done
    if [ ! -z "$INSTALL_LIST" ] ; then
        rpm $RPM_UNINSTALL_OPTS $INSTALL_LIST && return 0 || LogError Failed while installing packages $INSTALL_LIST
    fi
    return 1
}

CondInstall()
{
    INSTALL_LIST=
    for i in $@ ; do
        BASE=`basename $i .rpm`
        if rpm -q $BASE >/dev/null 2>&1 ; then
            INSTALL_LIST="$i $INSTALL_LIST"
        else
            Log Skipping $i, conditional package was not installed
        fi
    done
    Install $INSTALL_LIST
    return $?
}

RunScript()
{
    if [ -f "$1" ]; then
        chmod +x $1
        ./${1} || LogError script $1 failed
        Log Script $1 completed
    else
        LogError Missing script $1
        return $?
    fi
}

PromptForReboot()
{
	echo ""
	echo "Reboot required... Would you like to reboot now? (y/n):"
	read answer
	echo ""
	if [ "$answer" == "y" ] ; then
		echo "Rebooting your appliance now..."
		shutdown -r now
	else
		echo "OK. Please remember to reboot your appliance as soon as it is convenient."
	fi
	echo ""
}

echo ""
Log Hotfix Installation Started
echo ""

RPM_INSTALL_OPTS="-Uvh"

RPM_UNINSTALL_OPTS="--force -Uvh"

# BEGIN CUSTOMIZATION SECTION
ShowUsage()
{
	Log "install.sh [OPTIONS]"
	Log "	-r Rollback to previous settings."
	Log "	-h Show this message."
}

while getopts "rh" cmdopt $*
do
	case "$cmdopt" in
		"h")
		    MODE="usage"
		    break
		    ;;
		"r")
		    MODE="rollback"
		    break
		    ;;
		"*")
		    MODE="badargs"
		    break
		    ;;
	esac
done

Reload_NPU()
{
#Only needed for this (and similar) npu patches...

	#Reload the npu image
	echo "Resetting NPU..."
	/etc/iss/drivers/npuboot -r 
	#Sleep 25 sec for npu image reload to finish
	sleep 25
	echo "Finished resetting NPU."
	echo ""
}

#transform xml - need to supply 2 arguments
#1st arg - XML to transform
#2nd arg - transform to apply
TransformXML()
{
        #echo $#
	TEMP_XML=/tmp/tmp`basename $1`
	#backup target
	cp -f $1 $1.bak
	#do transform
	Xalan -o $TEMP_XML $1 $2 
	#evaluate return code
	if [ $? -eq 0 ]; then
		#test output
		eval $(echo `Xalan "$TEMP_XML" "$VALIDATOR" | awk '/The regex is/{regex=$4} END{print "export regex=\""regex"\""}'`)
		if [ "$2" == "$UPDATER" ] && [ "$regex" == "$NEWEXPRESSION" ]; then
			cp -f $TEMP_XML $1
			rm -f $TEMP_XML
		elif [ "$2" == "$REVERTER" ] && [ "$regex" == "$OLDEXPRESSION" ]; then
			cp -f $TEMP_XML $1
			rm -f $TEMP_XML
		else
			LogError Failed to read correct values from translated xml.
			cp -f $1.bak $1
		fi
	else
		LogError Failed to translate xml.
	fi
}

if [ $MODE == "usage" ]; then
	ShowUsage
	echo "No archiving of patch log due to usage argument."
	exit 1
else
	eval $(echo `issasApache -a si | awk ' /Last Firmware Update/{firm=$7} /Model Number/{model=substr($2,10)} END{print "export fwversion="firm" model="model} '`)

	echo "Checking Firmware Version..."
	echo $fwversion $model
	if [ -n "$fwversion" ]; then
		if [ $fwversion == 4.5 ]; then
			if [[ $model =~ "GX6.*" ]]; then 
				#Revert to original rpms
                                #Stop services
                                service issDaemon stop
                                service iss-watchdog stop
                                echo ""

				if [ $MODE == "install" ]; then
						#Install new rpms
						Install iss-provg-nileos-new.rpm
						Install iss-provg-host-msgtransport-new.rpm
				elif [ $MODE == "rollback" ]; then
						#Uninstall rpms
						Uninstall iss-provg-nileos-original.rpm
						Uninstall iss-provg-host-msgtransport-original.rpm
				fi
                                Reload_NPU
                                #Start Services
                                service iss-watchdog start
                                service issDaemon start
                                echo ""
				sync
			else
				echo "Incorrect target device for this patch. A GX6 Series appliance (FW 4.5) is required. Detected system is $model at $fwversion."
				exit 1
			fi
		else
			echo "Incorrect firmware version for this patch. Firmware 4.5 is required. Your firmware version is $fwversion."
			exit 1
		fi
	else
			echo "Please ensure the issDaemon is running prior to installing this patch."
			exit 1
	fi
fi

#END CUSTOMIZATION SECTION
echo ""
if [ $STATUS_ERR -ne 0 ] ; then
	Log "result_code = failed"
	Log "Hotfix Installation Failed"
	exit 1
elif [ $STATUS_WARN -ne 0 ] ; then
	Log "result_code = warning"
	Log "Hotfix installation completed with warnings"
else
	Log "result_code = success"
	Log "Hotfix Installation Completed Successfully"
fi

if [ -n "$fwversion" ] && [ -n "$model" ] && [ $fwversion == 4.5 ] && [[ $model =~ "GX6.*" ]]  ; then
	GetConfigValue "PatchArchiveDir=" PATCH_ARCHIVE_DIR
	if [ x"$NOT_FOUND" == x"$PATCH_ARCHIVE_DIR" ] ; then
		PATCH_ARCHIVE_DIR=$DEFAULT_PATCH_ARCHIVE_DIR
	fi

	# Test for existence of PATCH_ARCHIVE_DIR (create if doesn't exist)
	mkdir -p $PATCH_ARCHIVE_DIR
	if [ "$?" != "$SUCCESS" ] ; then
		echo "Unable to create $PATCH_ARCHIVE---exiting."
		exit $?
	fi

	TIMESTAMP=$(date +"%Y-%m-%d %H:%M:%S")

	# If archived results for patch already exists, append new results 
	# to bottom of file
	ACTION="unknown action"
	if [ "install" == "$MODE" ] ; then
		ACTION="Installed"
	elif [ "rollback" == "$MODE" ] ; then
		ACTION="Rolled Back"
	fi
	echo "$ACTION on $TIMESTAMP for FW $FW_VERSION" >> $PATCH_ARCHIVE_DIR/patch$PATCH_ID.results

	# Copy Patch's .info file to patch archive
	GetInfoFilename PATCH_INFO_FILE
	cp -u $PATCH_INFO_FILE $PATCH_ARCHIVE_DIR

	echo ""
	Log "Patch database updated."
	echo ""
	Log "Finished"
fi

exit $SUCCESS
